from tests_forms import TagCreateFormTest
from tests_forms import TagsCreateFormTest

from tests_models import O2OTagManagerTest
from tests_models import O2OTagTest

from tests_templatetags import BaseTaggedInObjectTest
from tests_templatetags import TaggedInObjectsNoDupsTest
from tests_templatetags import TaggedInObjectsTest

from tests_views import TagCreateViewTest
from tests_views import TagsCreateViewTest
