"""Use this app to tag objects like you tag your friends on facebook,
but using any objects as the tagger and the tagged. Think of a `User`
tagging a `Friend` in a `Photo` all of them being different models.

"""

__version__ = '0.2.3'
